'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var fs = _interopDefault(require('fs'));
var path = _interopDefault(require('path'));
var rollupPluginutils = require('rollup-pluginutils');
var cheerio = _interopDefault(require('cheerio'));
var pretty = _interopDefault(require('pretty'));

function loadXml(xml) {
  return cheerio.load(xml, { xmlMode: true });
}

function createSymbol(id, svg) {
  const element = loadXml(svg);
  const svgMarkup = element('svg');

  element('svg').replaceWith('<symbol/>');
  element('symbol').attr('id', id).attr('viewBox', svgMarkup.attr('viewBox')).append(svgMarkup.children());

  return element.html();
}

function createSprite(symbols, prettify) {
  const element = loadXml('<svg xmlns="http://www.w3.org/2000/svg"/>');

  symbols.forEach(symbol => {
    element('svg').append(symbol);
  });

  return prettify ? pretty(element.html()) : element.html();
}

function svgSprite(options = {}) {
  const filter = rollupPluginutils.createFilter(options.include, options.exclude);
  const prettify = options.prettify || false;
  const symbols = new Map();

  return {
    name: 'svg-sprite',
    transform: (code, id) => {
      if (!filter('id') || path.extname(id) !== '.svg') {
        return null;
      }

      const svgId = path.parse(id).name;

      symbols.set(svgId, createSymbol(svgId, code));

      return {
        code: '',
        map: { mappings: '' }
      };
    },
    onwrite: details => {
      if (symbols.size) {
        const outputFolderPath = options.outputFolder || path.dirname(details.file);

        fs.writeFileSync(`${outputFolderPath}/sprites.svg`, createSprite(symbols, prettify));
      }
    }
  };
}

module.exports = svgSprite;
